@@ -1,12 +1,9 @@
 import sys
 from numpy.testing import *
-set_package_path()
 from swig_ext import example2
-restore_path()
 
-class test_example2(ScipyTestCase):
-
-    def check_zoo(self):
+class TestExample2(TestCase):
+    def test_zoo(self):
         z = example2.Zoo()
         z.shut_up('Tiger')
         z.shut_up('Lion')
@@ -14,4 +11,4 @@ class test_example2(ScipyTestCase):
 
 
 if __name__ == "__main__":
-    ScipyTest().run()
+    run_module_suite()
