@@ -220,7 +220,7 @@ cppmacros['SWAP']="""\
 \ta = b;\\
 \tb = c;}
 """
-#cppmacros['ISCONTIGUOUS']='#define ISCONTIGUOUS(m) ((m)->flags & CONTIGUOUS)'
+#cppmacros['ISCONTIGUOUS']='#define ISCONTIGUOUS(m) ((m)->flags & NPY_CONTIGUOUS)'
 cppmacros['PRINTPYOBJERR']="""\
 #define PRINTPYOBJERR(obj)\\
 \tfprintf(stderr,\"#modulename#.error is related to \");\\
@@ -228,6 +228,8 @@ cppmacros['PRINTPYOBJERR']="""\
 \tfprintf(stderr,\"\\n\");
 """
 cppmacros['MINMAX']="""\
+#define max(a,b) ((a > b) ? (a) : (b))
+#define min(a,b) ((a < b) ? (a) : (b))
 #ifndef MAX
 #define MAX(a,b) ((a > b) ? (a) : (b))
 #endif
@@ -294,17 +296,17 @@ cppmacros['TRYPYARRAYTEMPLATE']="""\
                 case PyArray_FLOAT: *(float *)(arr->data)=*v; break;\\
                 case PyArray_CDOUBLE: *(double *)(arr->data)=*v; break;\\
                 case PyArray_CFLOAT: *(float *)(arr->data)=*v; break;\\
-                case PyArray_BOOL: *(Bool *)(arr->data)=(*v!=0); break;\\
+                case PyArray_BOOL: *(npy_bool *)(arr->data)=(*v!=0); break;\\
                 case PyArray_UBYTE: *(unsigned char *)(arr->data)=*v; break;\\
                 case PyArray_BYTE: *(signed char *)(arr->data)=*v; break;\\
                 case PyArray_SHORT: *(short *)(arr->data)=*v; break;\\
-                case PyArray_USHORT: *(ushort *)(arr->data)=*v; break;\\
-                case PyArray_UINT: *(uint *)(arr->data)=*v; break;\\
-                case PyArray_ULONG: *(ulong *)(arr->data)=*v; break;\\
-                case PyArray_LONGLONG: *(longlong *)(arr->data)=*v; break;\\
-                case PyArray_ULONGLONG: *(ulonglong *)(arr->data)=*v; break;\\
-                case PyArray_LONGDOUBLE: *(longdouble *)(arr->data)=*v; break;\\
-                case PyArray_CLONGDOUBLE: *(longdouble *)(arr->data)=*v; break;\\
+                case PyArray_USHORT: *(npy_ushort *)(arr->data)=*v; break;\\
+                case PyArray_UINT: *(npy_uint *)(arr->data)=*v; break;\\
+                case PyArray_ULONG: *(npy_ulong *)(arr->data)=*v; break;\\
+                case PyArray_LONGLONG: *(npy_longlong *)(arr->data)=*v; break;\\
+                case PyArray_ULONGLONG: *(npy_ulonglong *)(arr->data)=*v; break;\\
+                case PyArray_LONGDOUBLE: *(npy_longdouble *)(arr->data)=*v; break;\\
+                case PyArray_CLONGDOUBLE: *(npy_longdouble *)(arr->data)=*v; break;\\
                 case PyArray_OBJECT: (arr->descr->f->setitem)(pyobj_from_ ## ctype ## 1(*v),arr->data, arr); break;\\
         default: return -2;\\
         };\\
@@ -334,17 +336,17 @@ cppmacros['TRYCOMPLEXPYARRAYTEMPLATE']="""\
                 case PyArray_SHORT: *(short *)(arr->data)=(*v).r; break;\\
                 case PyArray_UBYTE: *(unsigned char *)(arr->data)=(*v).r; break;\\
                 case PyArray_BYTE: *(signed char *)(arr->data)=(*v).r; break;\\
-                case PyArray_BOOL: *(Bool *)(arr->data)=((*v).r!=0 && (*v).i!=0)); break;\\
+                case PyArray_BOOL: *(npy_bool *)(arr->data)=((*v).r!=0 && (*v).i!=0)); break;\\
                 case PyArray_UBYTE: *(unsigned char *)(arr->data)=(*v).r; break;\\
                 case PyArray_BYTE: *(signed char *)(arr->data)=(*v).r; break;\\
                 case PyArray_SHORT: *(short *)(arr->data)=(*v).r; break;\\
-                case PyArray_USHORT: *(ushort *)(arr->data)=(*v).r; break;\\
-                case PyArray_UINT: *(uint *)(arr->data)=(*v).r; break;\\
-                case PyArray_ULONG: *(ulong *)(arr->data)=(*v).r; break;\\
-                case PyArray_LONGLONG: *(longlong *)(arr->data)=(*v).r; break;\\
-                case PyArray_ULONGLONG: *(ulonglong *)(arr->data)=(*v).r; break;\\
-                case PyArray_LONGDOUBLE: *(longdouble *)(arr->data)=(*v).r; break;\\
-                case PyArray_CLONGDOUBLE: *(longdouble *)(arr->data)=(*v).r;*(longdouble *)(arr->data+sizeof(longdouble))=(*v).i;break;\\
+                case PyArray_USHORT: *(npy_ushort *)(arr->data)=(*v).r; break;\\
+                case PyArray_UINT: *(npy_uint *)(arr->data)=(*v).r; break;\\
+                case PyArray_ULONG: *(npy_ulong *)(arr->data)=(*v).r; break;\\
+                case PyArray_LONGLONG: *(npy_longlong *)(arr->data)=(*v).r; break;\\
+                case PyArray_ULONGLONG: *(npy_ulonglong *)(arr->data)=(*v).r; break;\\
+                case PyArray_LONGDOUBLE: *(npy_longdouble *)(arr->data)=(*v).r; break;\\
+                case PyArray_CLONGDOUBLE: *(npy_longdouble *)(arr->data)=(*v).r;*(npy_longdouble *)(arr->data+sizeof(npy_longdouble))=(*v).i;break;\\
                 case PyArray_OBJECT: (arr->descr->f->setitem)(pyobj_from_complex_ ## ctype ## 1((*v)),arr->data, arr); break;\\
                 default: return -2;\\
         };\\
@@ -407,13 +409,18 @@ cppmacros['GETSCALARFROMPYTUPLE']="""\
 \t}
 """
 
-needs['MEMCOPY']=['string.h']
+cppmacros['FAILNULL']="""\\
+#define FAILNULL(p) do {                                            \\
+    if ((p) == NULL) {                                              \\
+        PyErr_SetString(PyExc_MemoryError, "NULL pointer found");   \\
+        goto capi_fail;                                             \\
+    }                                                               \\
+} while (0)
+"""
+needs['MEMCOPY']=['string.h', 'FAILNULL']
 cppmacros['MEMCOPY']="""\
 #define MEMCOPY(to,from,n)\\
-\tif ((memcpy(to,from,n)) == NULL) {\\
-\t\tPyErr_SetString(PyExc_MemoryError, \"memcpy failed\");\\
-\t\tgoto capi_fail;\\
-\t}
+    do { FAILNULL(to); FAILNULL(from); (void)memcpy(to,from,n); } while (0)
 """
 cppmacros['STRINGMALLOC']="""\
 #define STRINGMALLOC(str,len)\\
@@ -425,26 +432,28 @@ cppmacros['STRINGMALLOC']="""\
 \t}
 """
 cppmacros['STRINGFREE']="""\
-#define STRINGFREE(str)\\
-\tif (!(str == NULL)) free(str);
+#define STRINGFREE(str) do {if (!(str == NULL)) free(str);} while (0)
 """
-needs['STRINGCOPYN']=['string.h']
+needs['STRINGCOPYN']=['string.h', 'FAILNULL']
 cppmacros['STRINGCOPYN']="""\
-#define STRINGCOPYN(to,from,n)\\
-\tif ((strncpy(to,from,sizeof(char)*(n))) == NULL) {\\
-\t\tPyErr_SetString(PyExc_MemoryError, \"strncpy failed\");\\
-\t\tgoto capi_fail;\\
-\t} else if (strlen(to)<(n)) {\\
-\t\tmemset((to)+strlen(to), ' ', (n)-strlen(to));\\
-\t} /* Padding with spaces instead of nulls. */
+#define STRINGCOPYN(to,from,buf_size)                           \\
+    do {                                                        \\
+        int _m = (buf_size);                                    \\
+        char *_to = (to);                                       \\
+        char *_from = (from);                                   \\
+        FAILNULL(_to); FAILNULL(_from);                         \\
+        (void)strncpy(_to, _from, sizeof(char)*_m);             \\
+        _to[_m-1] = '\\0';                                      \\
+        /* Padding with spaces instead of nulls */              \\
+        for (_m -= 2; _m >= 0 && _to[_m] == '\\0'; _m--) {      \\
+            _to[_m] = ' ';                                      \\
+        }                                                       \\
+    } while (0)
 """
-needs['STRINGCOPY']=['string.h']
+needs['STRINGCOPY']=['string.h', 'FAILNULL']
 cppmacros['STRINGCOPY']="""\
 #define STRINGCOPY(to,from)\\
-\tif ((strcpy(to,from)) == NULL) {\\
-\t\tPyErr_SetString(PyExc_MemoryError, \"strcpy failed\");\\
-\t\tgoto capi_fail;\\
-\t}
+    do { FAILNULL(to); FAILNULL(from); (void)strcpy(to,from); } while (0)
 """
 cppmacros['CHECKGENERIC']="""\
 #define CHECKGENERIC(check,tcheck,name) \\
@@ -503,8 +512,8 @@ static int calcarrindextr(int *i,PyArrayObject *arr) {
 \treturn ii;
 }"""
 cfuncs['forcomb']="""\
-static struct { int nd;intp *d;int *i,*i_tr,tr; } forcombcache;
-static int initforcomb(intp *dims,int nd,int tr) {
+static struct { int nd;npy_intp *d;int *i,*i_tr,tr; } forcombcache;
+static int initforcomb(npy_intp *dims,int nd,int tr) {
   int k;
   if (dims==NULL) return 0;
   if (nd<0) return 0;
@@ -567,7 +576,7 @@ fprintf(stderr,\"string_from_pyobj(str='%s',len=%d,inistr='%s',obj=%p)\\n\",(cha
 \t\tif (*len == -1)
 \t\t\t*len = strlen(inistr); /* Will this cause problems? */
 \t\tSTRINGMALLOC(*str,*len);
-\t\tSTRINGCOPYN(*str,inistr,*len);
+\t\tSTRINGCOPYN(*str,inistr,*len+1);
 \t\treturn 1;
 \t}
 \tif (PyArray_Check(obj)) {
@@ -580,7 +589,7 @@ fprintf(stderr,\"string_from_pyobj(str='%s',len=%d,inistr='%s',obj=%p)\\n\",(cha
 \t\tif (*len == -1)
 \t\t\t*len = (arr->descr->elsize)*PyArray_SIZE(arr);
 \t\tSTRINGMALLOC(*str,*len);
-\t\tSTRINGCOPYN(*str,arr->data,*len);
+\t\tSTRINGCOPYN(*str,arr->data,*len+1);
 \t\treturn 1;
 \t}
 \tif (PyString_Check(obj)) {
@@ -593,7 +602,7 @@ fprintf(stderr,\"string_from_pyobj(str='%s',len=%d,inistr='%s',obj=%p)\\n\",(cha
 \tif (*len == -1)
 \t\t*len = PyString_GET_SIZE(tmp);
 \tSTRINGMALLOC(*str,*len);
-\tSTRINGCOPYN(*str,PyString_AS_STRING(tmp),*len);
+\tSTRINGCOPYN(*str,PyString_AS_STRING(tmp),*len+1);
 \tPy_DECREF(tmp);
 \treturn 1;
 capi_fail:
@@ -750,7 +759,7 @@ static int long_double_from_pyobj(long_double* v,PyObject *obj,const char *errme
 \t\t\treturn 1;
 \t\t}
 \t\telse if (PyArray_Check(obj) && PyArray_TYPE(obj)==PyArray_LONGDOUBLE) {
-\t\t\t(*v) = *((longdouble *)PyArray_DATA(obj))
+\t\t\t(*v) = *((npy_longdouble *)PyArray_DATA(obj))
 \t\t\treturn 1;
 \t\t}
 \t}
@@ -823,8 +832,8 @@ static int complex_long_double_from_pyobj(complex_long_double* v,PyObject *obj,c
 \t\t\treturn 1;
 \t\t}
 \t\telse if (PyArray_Check(obj) && PyArray_TYPE(obj)==PyArray_CLONGDOUBLE) {
-\t\t\t(*v).r = ((clongdouble *)PyArray_DATA(obj))->real;
-\t\t\t(*v).i = ((clongdouble *)PyArray_DATA(obj))->imag;
+\t\t\t(*v).r = ((npy_clongdouble *)PyArray_DATA(obj))->real;
+\t\t\t(*v).i = ((npy_clongdouble *)PyArray_DATA(obj))->imag;
 \t\t\treturn 1;
 \t\t}
 \t}
@@ -847,13 +856,13 @@ static int complex_double_from_pyobj(complex_double* v,PyObject *obj,const char
 \t}
 \tif (PyArray_IsScalar(obj, ComplexFloating)) {
 \t\tif (PyArray_IsScalar(obj, CFloat)) {
-\t\t\tcfloat new;
+\t\t\tnpy_cfloat new;
 \t\t\tPyArray_ScalarAsCtype(obj, &new);
 \t\t\t(*v).r = (double)new.real;
 \t\t\t(*v).i = (double)new.imag;
 \t\t}
 \t\telse if (PyArray_IsScalar(obj, CLongDouble)) {
-\t\t\tclongdouble new;
+\t\t\tnpy_clongdouble new;
 \t\t\tPyArray_ScalarAsCtype(obj, &new);
 \t\t\t(*v).r = (double)new.real;
 \t\t\t(*v).i = (double)new.imag;
@@ -872,8 +881,8 @@ static int complex_double_from_pyobj(complex_double* v,PyObject *obj,const char
 \t\t\tarr = PyArray_FromScalar(obj, PyArray_DescrFromType(PyArray_CDOUBLE));
 \t\t}
 \t\tif (arr==NULL) return 0;
-\t\t(*v).r = ((cdouble *)PyArray_DATA(arr))->real;
-\t\t(*v).i = ((cdouble *)PyArray_DATA(arr))->imag;
+\t\t(*v).r = ((npy_cdouble *)PyArray_DATA(arr))->real;
+\t\t(*v).i = ((npy_cdouble *)PyArray_DATA(arr))->imag;
 \t\treturn 1;
 \t}
 \t/* Python does not provide PyNumber_Complex function :-( */
@@ -1060,28 +1069,39 @@ def append_needs(need,flag=1):
             append_needs(n,flag)
     elif type(need)==types.StringType:
         if not need: return
-        if includes0.has_key(need): n = 'includes0'
-        elif includes.has_key(need): n = 'includes'
-        elif typedefs.has_key(need): n = 'typedefs'
-        elif typedefs_generated.has_key(need): n = 'typedefs_generated'
-        elif cppmacros.has_key(need): n = 'cppmacros'
-        elif cfuncs.has_key(need): n = 'cfuncs'
-        elif callbacks.has_key(need): n = 'callbacks'
-        elif f90modhooks.has_key(need): n = 'f90modhooks'
-        elif commonhooks.has_key(need): n = 'commonhooks'
+        if need in includes0:
+            n = 'includes0'
+        elif need in includes:
+            n = 'includes'
+        elif need in typedefs:
+            n = 'typedefs'
+        elif need in typedefs_generated:
+            n = 'typedefs_generated'
+        elif need in cppmacros:
+            n = 'cppmacros'
+        elif need in cfuncs:
+            n = 'cfuncs'
+        elif need in callbacks:
+            n = 'callbacks'
+        elif need in f90modhooks:
+            n = 'f90modhooks'
+        elif need in commonhooks:
+            n = 'commonhooks'
         else:
             errmess('append_needs: unknown need %s\n'%(`need`))
             return
         if need in outneeds[n]: return
         if flag:
             tmp={}
-            if needs.has_key(need):
+            if need in needs:
                 for nn in needs[need]:
                     t=append_needs(nn,0)
                     if type(t)==types.DictType:
                         for nnn in t.keys():
-                            if tmp.has_key(nnn): tmp[nnn]=tmp[nnn]+t[nnn]
-                            else: tmp[nnn]=t[nnn]
+                            if nnn in tmp:
+                                tmp[nnn]=tmp[nnn]+t[nnn]
+                            else:
+                                tmp[nnn]=t[nnn]
             for nn in tmp.keys():
                 for nnn in tmp[nn]:
                     if nnn not in outneeds[nn]:
@@ -1089,14 +1109,17 @@ def append_needs(need,flag=1):
             outneeds[n].append(need)
         else:
             tmp={}
-            if needs.has_key(need):
+            if need in needs:
                 for nn in needs[need]:
                     t=append_needs(nn,flag)
                     if type(t)==types.DictType:
                         for nnn in t.keys():
-                            if tmp.has_key(nnn): tmp[nnn]=t[nnn]+tmp[nnn]
-                            else: tmp[nnn]=t[nnn]
-            if not tmp.has_key(n): tmp[n]=[]
+                            if nnn in tmp:
+                                tmp[nnn]=t[nnn]+tmp[nnn]
+                            else:
+                                tmp[nnn]=t[nnn]
+            if n not in tmp:
+                tmp[n]=[]
             tmp[n].append(need)
             return tmp
     else:
@@ -1109,7 +1132,7 @@ def get_needs():
         out=[]
         saveout=copy.copy(outneeds[n])
         while len(outneeds[n])>0:
-            if not needs.has_key(outneeds[n][0]):
+            if outneeds[n][0] not in needs:
                 out.append(outneeds[n][0])
                 del outneeds[n][0]
             else:
@@ -1129,6 +1152,7 @@ def get_needs():
                 out=out+saveout
                 break
             saveout=copy.copy(outneeds[n])
-        if out==[]: out=[n]
+        if out==[]:
+            out=[n]
         res[n]=out
     return res
