@@ -1,18 +1,22 @@
 """Lite version of scipy.linalg.
 
-This module is a lite version of the linalg.py module in SciPy which contains
-high-level Python interface to the LAPACK library.  The lite version
-only accesses the following LAPACK functions: dgesv, zgesv, dgeev,
-zgeev, dgesdd, zgesdd, dgelsd, zgelsd, dsyevd, zheevd, dgetrf, dpotrf.
+Notes
+-----
+This module is a lite version of the linalg.py module in SciPy which
+contains high-level Python interface to the LAPACK library.  The lite
+version only accesses the following LAPACK functions: dgesv, zgesv,
+dgeev, zgeev, dgesdd, zgesdd, dgelsd, zgelsd, dsyevd, zheevd, dgetrf,
+zgetrf, dpotrf, zpotrf, dgeqrf, zgeqrf, zungqr, dorgqr.
 """
 
-__all__ = ['solve', 'tensorsolve', 'tensorinv',
+__all__ = ['matrix_power', 'solve', 'tensorsolve', 'tensorinv',
            'inv', 'cholesky',
            'eigvals',
            'eigvalsh', 'pinv',
            'det', 'svd',
            'eig', 'eigh','lstsq', 'norm',
            'qr',
+           'cond',
            'LinAlgError'
            ]
 
@@ -23,6 +27,7 @@ from numpy.core import array, asarray, zeros, empty, transpose, \
         isfinite, size
 from numpy.lib import triu
 from numpy.linalg import lapack_lite
+from numpy.core.defmatrix import matrix_power, matrix
 
 fortran_int = intc
 
@@ -87,14 +92,7 @@ def _commonType(*arrays):
         t = double
     return t, result_type
 
-def _castCopyAndTranspose(type, *arrays):
-    if len(arrays) == 1:
-        return transpose(arrays[0]).astype(type)
-    else:
-        return [transpose(a).astype(type) for a in arrays]
-
-# _fastCopyAndTranpose is an optimized version of _castCopyAndTranspose.
-# It assumes the input is 2D (as all the calls in here are).
+# _fastCopyAndTranpose assumes the input is 2D (as all the calls in here are).
 
 _fastCT = fastCopyAndTranspose
 
@@ -131,18 +129,45 @@ def _assertNonEmpty(*arrays):
         if size(a) == 0:
             raise LinAlgError("Arrays cannot be empty")
 
+
 # Linear equations
 
 def tensorsolve(a, b, axes=None):
-    """Solves the tensor equation a x = b for x
-
-    where it is assumed that all the indices of x are summed over in
-    the product.
-
-    a can be N-dimensional.  x will have the dimensions of A subtracted from
-    the dimensions of b.
     """
-    a = asarray(a)
+    Solve the tensor equation a x = b for x
+
+    It is assumed that all indices of x are summed over in the product,
+    together with the rightmost indices of a, similarly as in
+    tensordot(a, x, axes=len(b.shape)).
+
+    Parameters
+    ----------
+    a : array_like, shape b.shape+Q
+        Coefficient tensor. Shape Q of the rightmost indices of a must
+        be such that a is 'square', ie., prod(Q) == prod(b.shape).
+    b : array_like, any shape
+        Right-hand tensor.
+    axes : tuple of integers
+        Axes in a to reorder to the right, before inversion.
+        If None (default), no reordering is done.
+
+    Returns
+    -------
+    x : array, shape Q
+
+    Examples
+    --------
+    >>> a = np.eye(2*3*4)
+    >>> a.shape = (2*3,4,  2,3,4)
+    >>> b = np.random.randn(2*3,4)
+    >>> x = np.linalg.tensorsolve(a, b)
+    >>> x.shape
+    (2, 3, 4)
+    >>> np.allclose(np.tensordot(a, x, axes=3), b)
+    True
+
+    """
+    a,wrap = _makearray(a)
     b = asarray(b)
     an = a.ndim
 
@@ -160,13 +185,48 @@ def tensorsolve(a, b, axes=None):
 
     a = a.reshape(-1, prod)
     b = b.ravel()
-    res = solve(a, b)
+    res = wrap(solve(a, b))
     res.shape = oldshape
     return res
 
 def solve(a, b):
-    """Return the solution of a*x = b
     """
+    Solve the equation ``a x = b`` for ``x``.
+
+    Parameters
+    ----------
+    a : array_like, shape (M, M)
+        Input equation coefficients.
+    b : array_like, shape (M,)
+        Equation target values.
+
+    Returns
+    -------
+    x : array, shape (M,)
+
+    Raises
+    ------
+    LinAlgError
+        If `a` is singular or not square.
+
+    Examples
+    --------
+    Solve the system of equations ``3 * x0 + x1 = 9`` and ``x0 + 2 * x1 = 8``:
+
+    >>> a = np.array([[3,1], [1,2]])
+    >>> b = np.array([9,8])
+    >>> x = np.linalg.solve(a, b)
+    >>> x
+    array([ 2.,  3.])
+
+    Check that the solution is correct:
+
+    >>> (np.dot(a, x) == b).all()
+    True
+
+    """
+    a, _ = _makearray(a)
+    b, wrap = _makearray(b)
     one_eq = len(b.shape) == 1
     if one_eq:
         b = b[:, newaxis]
@@ -188,41 +248,56 @@ def solve(a, b):
     if results['info'] > 0:
         raise LinAlgError, 'Singular matrix'
     if one_eq:
-        return b.ravel().astype(result_t)
+        return wrap(b.ravel().astype(result_t))
     else:
-        return b.transpose().astype(result_t)
+        return wrap(b.transpose().astype(result_t))
 
 
 def tensorinv(a, ind=2):
-    """Find the 'inverse' of a N-d array
+    """
+    Find the 'inverse' of a N-d array
 
-    ind must be a positive integer specifying
-    how many indices at the front of the array are involved
-    in the inverse sum.
+    The result is an inverse corresponding to the operation
+    tensordot(a, b, ind), ie.,
 
-    the result is ainv with shape a.shape[ind:] + a.shape[:ind]
+        x == tensordot(tensordot(tensorinv(a), a, ind), x, ind)
+          == tensordot(tensordot(a, tensorinv(a), ind), x, ind)
 
-    tensordot(ainv, a, ind) is an identity operator
+    for all x (up to floating-point accuracy).
 
-    and so is
+    Parameters
+    ----------
+    a : array_like
+        Tensor to 'invert'. Its shape must 'square', ie.,
+        prod(a.shape[:ind]) == prod(a.shape[ind:])
+    ind : integer > 0
+        How many of the first indices are involved in the inverse sum.
 
-    tensordot(a, ainv, a.shape-newind)
+    Returns
+    -------
+    b : array, shape a.shape[:ind]+a.shape[ind:]
 
-    Example:
+    Raises LinAlgError if a is singular or not square
 
-       a = rand(4,6,8,3)
-       ainv = tensorinv(a)
-       # ainv.shape is (8,3,4,6)
-       # suppose b has shape (4,6)
-       tensordot(ainv, b) # produces same (8,3)-shaped output as
-       tensorsolve(a, b)
+    Examples
+    --------
+    >>> a = np.eye(4*6)
+    >>> a.shape = (4,6,8,3)
+    >>> ainv = np.linalg.tensorinv(a, ind=2)
+    >>> ainv.shape
+    (8, 3, 4, 6)
+    >>> b = np.random.randn(4,6)
+    >>> np.allclose(np.tensordot(ainv, b), np.linalg.tensorsolve(a, b))
+    True
 
-       a = rand(24,8,3)
-       ainv = tensorinv(a,1)
-       # ainv.shape is (8,3,24)
-       # suppose b has shape (24,)
-       tensordot(ainv, b, 1)  # produces the same (8,3)-shaped output as
-       tensorsolve(a, b)
+    >>> a = np.eye(4*6)
+    >>> a.shape = (24,8,3)
+    >>> ainv = np.linalg.tensorinv(a, ind=1)
+    >>> ainv.shape
+    (8, 3, 24)
+    >>> b = np.random.randn(24)
+    >>> np.allclose(np.tensordot(ainv, b, 1), np.linalg.tensorsolve(a, b))
+    True
 
     """
     a = asarray(a)
@@ -242,12 +317,91 @@ def tensorinv(a, ind=2):
 # Matrix inversion
 
 def inv(a):
+    """
+    Compute the inverse of a matrix.
+
+    Parameters
+    ----------
+    a : array_like, shape (M, M)
+        Matrix to be inverted
+
+    Returns
+    -------
+    ainv : ndarray, shape (M, M)
+        Inverse of the matrix `a`
+
+    Raises
+    ------
+    LinAlgError
+        If `a` is singular or not square.
+
+    Examples
+    --------
+    >>> a = np.array([[1., 2.], [3., 4.]])
+    >>> np.linalg.inv(a)
+    array([[-2. ,  1. ],
+           [ 1.5, -0.5]])
+    >>> np.dot(a, np.linalg.inv(a))
+    array([[ 1.,  0.],
+           [ 0.,  1.]])
+
+    """
     a, wrap = _makearray(a)
     return wrap(solve(a, identity(a.shape[0], dtype=a.dtype)))
 
+
 # Cholesky decomposition
 
 def cholesky(a):
+    """
+    Cholesky decomposition.
+
+    Return the Cholesky decomposition, :math:`A = L L^*` of a Hermitian
+    positive-definite matrix :math:`A`.
+
+    Parameters
+    ----------
+    a : array_like, shape (M, M)
+        Hermitian (symmetric, if it is real) and positive definite
+        input matrix.
+
+    Returns
+    -------
+    L : array_like, shape (M, M)
+        Lower-triangular Cholesky factor of A.
+
+    Raises
+    ------
+    LinAlgError
+       If the decomposition fails.
+
+    Notes
+    -----
+    The Cholesky decomposition is often used as a fast way of solving
+
+    .. math:: A \\mathbf{x} = \\mathbf{b}.
+
+    First, we solve for :math:`\\mathbf{y}` in
+
+    .. math:: L \\mathbf{y} = \\mathbf{b},
+
+    and then for :math:`\\mathbf{x}` in
+
+    .. math:: L^* \\mathbf{x} = \\mathbf{y}.
+
+    Examples
+    --------
+    >>> A = np.array([[1,-2j],[2j,5]])
+    >>> L = np.linalg.cholesky(A)
+    >>> L
+    array([[ 1.+0.j,  0.+0.j],
+           [ 0.+2.j,  1.+0.j]])
+    >>> np.dot(L, L.T.conj())
+    array([[ 1.+0.j,  0.-2.j],
+           [ 0.+2.j,  5.+0.j]])
+
+    """
+    a, wrap = _makearray(a)
     _assertRank2(a)
     _assertSquareness(a)
     t, result_t = _commonType(a)
@@ -264,19 +418,65 @@ def cholesky(a):
         Cholesky decomposition cannot be computed'
     s = triu(a, k=0).transpose()
     if (s.dtype != result_t):
-        return s.astype(result_t)
-    return s
+        s = s.astype(result_t)
+    return wrap(s)
 
 # QR decompostion
 
 def qr(a, mode='full'):
-    """cacluates A=QR, Q orthonormal, R upper triangular
-
-    mode:  'full' --> (Q,R)
-           'r'    --> R
-           'economic' --> A2 where the diagonal + upper triangle
-                   part of A2 is R. This is faster if you only need R
     """
+    Compute QR decomposition of a matrix.
+
+    Calculate the decomposition :math:`A = Q R` where Q is orthonormal
+    and R upper triangular.
+
+    Parameters
+    ----------
+    a : array_like, shape (M, N)
+        Matrix to be decomposed
+    mode : {'full', 'r', 'economic'}
+        Determines what information is to be returned. 'full' is the default.
+        Economic mode is slightly faster if only R is needed.
+
+    Returns
+    -------
+    mode = 'full'
+    Q : double or complex array, shape (M, K)
+    R : double or complex array, shape (K, N)
+        Size K = min(M, N)
+
+    mode = 'r'
+    R : double or complex array, shape (K, N)
+
+    mode = 'economic'
+    A2 : double or complex array, shape (M, N)
+        The diagonal and the upper triangle of A2 contains R,
+        while the rest of the matrix is undefined.
+
+    If a is a matrix, so are all the return values.
+
+    Raises LinAlgError if decomposition fails
+
+    Notes
+    -----
+    This is an interface to the LAPACK routines dgeqrf, zgeqrf,
+    dorgqr, and zungqr.
+
+    Examples
+    --------
+    >>> a = np.random.randn(9, 6)
+    >>> q, r = np.linalg.qr(a)
+    >>> np.allclose(a, np.dot(q, r))
+    True
+    >>> r2 = np.linalg.qr(a, mode='r')
+    >>> r3 = np.linalg.qr(a, mode='economic')
+    >>> np.allclose(r, r2)
+    True
+    >>> np.allclose(r, np.triu(r3[:6,:6], k=0))
+    True
+
+    """
+    a, wrap = _makearray(a)
     _assertRank2(a)
     m, n = a.shape
     t, result_t = _commonType(a)
@@ -345,46 +545,53 @@ def qr(a, mode='full'):
 
     q = _fastCopyAndTranspose(result_t, a[:mn,:])
 
-    return q, r
+    return wrap(q), wrap(r)
 
 
 # Eigenvalues
 
 
 def eigvals(a):
-    """Compute the eigenvalues of the general 2-d array a.
+    """
+    Compute the eigenvalues of a general matrix.
 
-    A simple interface to the LAPACK routines dgeev and zgeev that sets the
-    flags to return only the eigenvalues of general real and complex arrays
-    respectively.
+    Parameters
+    ----------
+    a : array_like, shape (M, M)
+        A complex or real matrix whose eigenvalues and eigenvectors
+        will be computed.
 
-    :Parameters:
-
-        a : 2-d array
-            A complex or real 2-d array whose eigenvalues and eigenvectors
-            will be computed.
-
-    :Returns:
-
-        w : 1-d double or complex array
-            The eigenvalues. The eigenvalues are not necessarily ordered, nor
-            are they necessarily real for real matrices.
-
-    :SeeAlso:
-
-        - eig : eigenvalues and right eigenvectors of general arrays
-        - eigvalsh : eigenvalues of symmetric or Hemitiean arrays.
-        - eigh : eigenvalues and eigenvectors of symmetric/Hermitean arrays.
-
-    :Notes:
+    Returns
     -------
+    w : ndarray, shape (M,)
+        The eigenvalues, each repeated according to its multiplicity.
+        They are not necessarily ordered, nor are they necessarily
+        real for real matrices.
 
-        The number w is an eigenvalue of a if there exists a vector v
-        satisfying the equation dot(a,v) = w*v. Alternately, if w is a root of
-        the characteristic equation det(a - w[i]*I) = 0, where det is the
-        determinant and I is the identity matrix.
+    Raises
+    ------
+    LinAlgError
+        If the eigenvalue computation does not converge.
+
+    See Also
+    --------
+    eig : eigenvalues and right eigenvectors of general arrays
+    eigvalsh : eigenvalues of symmetric or Hemitiean arrays.
+    eigh : eigenvalues and eigenvectors of symmetric/Hermitean arrays.
+
+    Notes
+    -----
+    This is a simple interface to the LAPACK routines dgeev and zgeev
+    that sets the flags to return only the eigenvalues of general real
+    and complex arrays respectively.
+
+    The number w is an eigenvalue of a if there exists a vector v
+    satisfying the equation dot(a,v) = w*v. Alternately, if w is a root of
+    the characteristic equation det(a - w[i]*I) = 0, where det is the
+    determinant and I is the identity matrix.
 
     """
+    a, wrap = _makearray(a)
     _assertRank2(a)
     _assertSquareness(a)
     _assertFinite(a)
@@ -429,44 +636,49 @@ def eigvals(a):
 
 
 def eigvalsh(a, UPLO='L'):
-    """Compute the eigenvalues of the symmetric or Hermitean 2-d array a.
+    """
+    Compute the eigenvalues of a Hermitean or real symmetric matrix.
 
-    A simple interface to the LAPACK routines dsyevd and zheevd that sets the
-    flags to return only the eigenvalues of real symmetric and complex
-    Hermetian arrays respectively.
+    Parameters
+    ----------
+    a : array_like, shape (M, M)
+        A complex or real matrix whose eigenvalues and eigenvectors
+        will be computed.
+    UPLO : {'L', 'U'}, optional
+        Specifies whether the calculation is done with data from the
+        lower triangular part of `a` ('L', default) or the upper triangular
+        part ('U').
 
-    :Parameters:
-
-        a : 2-d array
-            A complex or real 2-d array whose eigenvalues and eigenvectors
-            will be computed.
-
-        UPLO : string
-            Specifies whether the pertinent array date is taken from the upper
-            or lower triangular part of a. Possible values are 'L', and 'U' for
-            upper and lower respectively. Default is 'L'.
-
-    :Returns:
-
-        w : 1-d double array
-            The eigenvalues. The eigenvalues are not necessarily ordered.
-
-    :SeeAlso:
-
-        - eigh : eigenvalues and eigenvectors of symmetric/Hermitean arrays.
-        - eigvals : eigenvalues of general real or complex arrays.
-        - eig : eigenvalues and eigenvectors of general real or complex arrays.
-
-    :Notes:
+    Returns
     -------
+    w : ndarray, shape (M,)
+        The eigenvalues, each repeated according to its multiplicity.
+        They are not necessarily ordered.
 
-        The number w is an eigenvalue of a if there exists a vector v
-        satisfying the equation dot(a,v) = w*v. Alternately, if w is a root of
-        the characteristic equation det(a - w[i]*I) = 0, where det is the
-        determinant and I is the identity matrix. The eigenvalues of real
-        symmetric or complex Hermitean matrices are always real.
+    Raises
+    ------
+    LinAlgError
+        If the eigenvalue computation does not converge.
+
+    See Also
+    --------
+    eigh : eigenvalues and eigenvectors of symmetric/Hermitean arrays.
+    eigvals : eigenvalues of general real or complex arrays.
+    eig : eigenvalues and eigenvectors of general real or complex arrays.
+
+    Notes
+    -----
+    This is a simple interface to the LAPACK routines dsyevd and
+    zheevd that sets the flags to return only the eigenvalues of real
+    symmetric and complex Hermetian arrays respectively.
+
+    The number w is an eigenvalue of a if there exists a vector v
+    satisfying the equation dot(a,v) = w*v. Alternately, if w is a root of
+    the characteristic equation det(a - w[i]*I) = 0, where det is the
+    determinant and I is the identity matrix.
 
     """
+    a, wrap = _makearray(a)
     _assertRank2(a)
     _assertSquareness(a)
     t, result_t = _commonType(a)
@@ -515,51 +727,55 @@ def _convertarray(a):
 
 
 def eig(a):
-    """Eigenvalues and right eigenvectors of a general matrix.
+    """
+    Compute eigenvalues and right eigenvectors of an array.
 
-    A simple interface to the LAPACK routines dgeev and zgeev that compute the
-    eigenvalues and eigenvectors of general real and complex arrays
-    respectively.
+    Parameters
+    ----------
+    a : array_like, shape (M, M)
+        A complex or real 2-D array.
 
-    :Parameters:
-
-        a : 2-d array
-            A complex or real 2-d array whose eigenvalues and eigenvectors
-            will be computed.
-
-    :Returns:
-
-        w : 1-d double or complex array
-            The eigenvalues. The eigenvalues are not necessarily ordered, nor
-            are they necessarily real for real matrices.
-
-        v : 2-d double or complex double array.
-            The normalized eigenvector corresponding to the eigenvalue w[i] is
-            the column v[:,i].
-
-    :SeeAlso:
-
-        - eigvalsh : eigenvalues of symmetric or Hemitiean arrays.
-        - eig : eigenvalues and right eigenvectors for non-symmetric arrays
-        - eigvals : eigenvalues of non-symmetric array.
-
-    :Notes:
+    Returns
     -------
+    w : ndarray, shape (M,)
+        The eigenvalues, each repeated according to its multiplicity.
+        The eigenvalues are not necessarily ordered, nor are they
+        necessarily real for real matrices.
+    v : ndarray, shape (M, M)
+        The normalized eigenvector corresponding to the eigenvalue ``w[i]`` is
+        the column ``v[:,i]``.
 
-        The number w is an eigenvalue of a if there exists a vector v
-        satisfying the equation dot(a,v) = w*v. Alternately, if w is a root of
-        the characteristic equation det(a - w[i]*I) = 0, where det is the
-        determinant and I is the identity matrix. The arrays a, w, and v
-        satisfy the equation dot(a,v[i]) = w[i]*v[:,i].
+    Raises
+    ------
+    LinAlgError
+        If the eigenvalue computation does not converge.
 
-        The array v of eigenvectors may not be of maximum rank, that is, some
-        of the columns may be dependent, although roundoff error may obscure
-        that fact. If the eigenvalues are all different, then theoretically the
-        eigenvectors are independent. Likewise, the matrix of eigenvectors is
-        unitary if the matrix a is normal, i.e., if dot(a, a.H) = dot(a.H, a).
+    See Also
+    --------
+    eigvalsh : eigenvalues of symmetric or Hemitiean arrays.
+    eig : eigenvalues and right eigenvectors for non-symmetric arrays
+    eigvals : eigenvalues of non-symmetric array.
 
-        The left and right eigenvectors are not necessarily the (Hemitian)
-        transposes of each other.
+    Notes
+    -----
+    This is a simple interface to the LAPACK routines dgeev and zgeev
+    that compute the eigenvalues and eigenvectors of general real and
+    complex arrays respectively.
+
+    The number `w` is an eigenvalue of a if there exists a vector `v`
+    satisfying the equation ``dot(a,v) = w*v``. Alternately, if `w` is a root of
+    the characteristic equation ``det(a - w[i]*I) = 0``, where `det` is the
+    determinant and `I` is the identity matrix. The arrays `a`, `w`, and `v`
+    satisfy the equation ``dot(a,v[i]) = w[i]*v[:,i]``.
+
+    The array `v` of eigenvectors may not be of maximum rank, that is, some
+    of the columns may be dependent, although roundoff error may obscure
+    that fact. If the eigenvalues are all different, then theoretically the
+    eigenvectors are independent. Likewise, the matrix of eigenvectors is
+    unitary if the matrix `a` is normal, i.e., if ``dot(a, a.H) = dot(a.H, a)``.
+
+    The left and right eigenvectors are not necessarily the (Hermitian)
+    transposes of each other.
 
     """
     a, wrap = _makearray(a)
@@ -617,48 +833,50 @@ def eig(a):
 
 
 def eigh(a, UPLO='L'):
-    """Compute eigenvalues for a Hermitian-symmetric matrix.
+    """
+    Eigenvalues and eigenvectors of a Hermitian or real symmetric matrix.
 
+    Parameters
+    ----------
+    a : array_like, shape (M, M)
+        A complex Hermitian or symmetric real matrix.
+    UPLO : {'L', 'U'}, optional
+        Specifies whether the calculation is done with data from the
+        lower triangular part of `a` ('L', default) or the upper triangular
+        part ('U').
+
+    Returns
+    -------
+    w : ndarray, shape (M,)
+        The eigenvalues. The eigenvalues are not necessarily ordered.
+    v : ndarray, shape (M, M)
+        The normalized eigenvector corresponding to the eigenvalue w[i] is
+        the column v[:,i].
+
+    Raises
+    ------
+    LinAlgError
+        If the eigenvalue computation does not converge.
+
+    See Also
+    --------
+    eigvalsh : eigenvalues of symmetric or Hemitiean arrays.
+    eig : eigenvalues and right eigenvectors for non-symmetric arrays
+    eigvals : eigenvalues of non-symmetric array.
+
+    Notes
+    -----
     A simple interface to the LAPACK routines dsyevd and zheevd that compute
     the eigenvalues and eigenvectors of real symmetric and complex Hermitian
     arrays respectively.
 
-    :Parameters:
-
-        a : 2-d array
-            A complex Hermitian or symmetric real 2-d array whose eigenvalues
-            and eigenvectors will be computed.
-
-        UPLO : string
-            Specifies whether the pertinent array date is taken from the upper
-            or lower triangular part of a. Possible values are 'L', and 'U'.
-            Default is 'L'.
-
-    :Returns:
-
-        w : 1-d double array
-            The eigenvalues. The eigenvalues are not necessarily ordered.
-
-        v : 2-d double or complex double array, depending on input array type
-            The normalized eigenvector corresponding to the eigenvalue w[i] is
-            the column v[:,i].
-
-    :SeeAlso:
-
-        - eigvalsh : eigenvalues of symmetric or Hemitiean arrays.
-        - eig : eigenvalues and right eigenvectors for non-symmetric arrays
-        - eigvals : eigenvalues of non-symmetric array.
-
-    :Notes:
-    -------
-
-        The number w is an eigenvalue of a if there exists a vector v
-        satisfying the equation dot(a,v) = w*v. Alternately, if w is a root of
-        the characteristic equation det(a - w[i]*I) = 0, where det is the
-        determinant and I is the identity matrix. The eigenvalues of real
-        symmetric or complex Hermitean matrices are always real. The array v
-        of eigenvectors is unitary and a, w, and v satisfy the equation
-        dot(a,v[i]) = w[i]*v[:,i].
+    The number w is an eigenvalue of a if there exists a vector v
+    satisfying the equation dot(a,v) = w*v. Alternately, if w is a root of
+    the characteristic equation det(a - w[i]*I) = 0, where det is the
+    determinant and I is the identity matrix. The eigenvalues of real
+    symmetric or complex Hermitean matrices are always real. The array v
+    of eigenvectors is unitary and a, w, and v satisfy the equation
+    dot(a,v[i]) = w[i]*v[:,i].
 
     """
     a, wrap = _makearray(a)
@@ -705,21 +923,63 @@ def eigh(a, UPLO='L'):
 # Singular value decomposition
 
 def svd(a, full_matrices=1, compute_uv=1):
-    """Singular Value Decomposition.
+    """
+    Singular Value Decomposition.
 
-    u,s,vh = svd(a)
+    Factorizes the matrix `a` into two unitary matrices, ``U`` and ``Vh``,
+    and a 1-dimensional array of singular values, ``s`` (real, non-negative),
+    such that ``a == U S Vh``, where ``S`` is the diagonal
+    matrix ``np.diag(s)``.
 
-    If a is an M x N array, then the svd produces a factoring of the array
-    into two unitary (orthogonal) 2-d arrays u (MxM) and vh (NxN) and a
-    min(M,N)-length array of singular values such that
+    Parameters
+    ----------
+    a : array_like, shape (M, N)
+        Matrix to decompose
+    full_matrices : boolean, optional
+        If True (default), ``U`` and ``Vh`` are shaped
+        ``(M,M)`` and ``(N,N)``.  Otherwise, the shapes are
+        ``(M,K)`` and ``(K,N)``, where ``K = min(M,N)``.
+    compute_uv : boolean
+        Whether to compute ``U`` and ``Vh`` in addition to ``s``.
+        True by default.
 
-                     a == dot(u,dot(S,vh))
+    Returns
+    -------
+    U : ndarray, shape (M, M) or (M, K) depending on `full_matrices`
+        Unitary matrix.
+    s :  ndarray, shape (K,) where ``K = min(M, N)``
+        The singular values, sorted so that ``s[i] >= s[i+1]``.
+    Vh : ndarray, shape (N,N) or (K,N) depending on `full_matrices`
+        Unitary matrix.
 
-    where S is an MxN array of zeros whose main diagonal is s.
+    Raises
+    ------
+    LinAlgError
+        If SVD computation does not converge.
+
+    Notes
+    -----
+    If `a` is a matrix (in contrast to an ndarray), then so are all
+    the return values.
+
+    Examples
+    --------
+    >>> a = np.random.randn(9, 6) + 1j*np.random.randn(9, 6)
+    >>> U, s, Vh = np.linalg.svd(a)
+    >>> U.shape, Vh.shape, s.shape
+    ((9, 9), (6, 6), (6,))
+
+    >>> U, s, Vh = np.linalg.svd(a, full_matrices=False)
+    >>> U.shape, Vh.shape, s.shape
+    ((9, 6), (6, 6), (6,))
+    >>> S = np.diag(s)
+    >>> np.allclose(a, np.dot(U, np.dot(S, Vh)))
+    True
+
+    >>> s2 = np.linalg.svd(a, compute_uv=False)
+    >>> np.allclose(s, s2)
+    True
 
-    if compute_uv == 0, then return only the singular values
-    if full_matrices == 0, then only part of either u or vh is
-                           returned so that it is MxN
     """
     a, wrap = _makearray(a)
     _assertRank2(a)
@@ -779,14 +1039,87 @@ def svd(a, full_matrices=1, compute_uv=1):
     else:
         return s
 
+def cond(x, p=None):
+    """
+    Compute the condition number of a matrix.
+
+    The condition number of `x` is the norm of `x` times the norm
+    of the inverse of `x`.  The norm can be the usual L2
+    (root-of-sum-of-squares) norm or a number of other matrix norms.
+
+    Parameters
+    ----------
+    x : array_like, shape (M, N)
+        The matrix whose condition number is sought.
+    p : {None, 1, -1, 2, -2, inf, -inf, 'fro'}
+        Order of the norm:
+
+        =====  ============================
+        p      norm for matrices
+        =====  ============================
+        None   2-norm, computed directly using the SVD
+        'fro'  Frobenius norm
+        inf    max(sum(abs(x), axis=1))
+        -inf   min(sum(abs(x), axis=1))
+        1      max(sum(abs(x), axis=0))
+        -1     min(sum(abs(x), axis=0))
+        2      2-norm (largest sing. value)
+        -2     smallest singular value
+        =====  ============================
+
+    Returns
+    -------
+    c : float
+        The condition number of the matrix. May be infinite.
+
+    """
+    x = asarray(x) # in case we have a matrix
+    if p is None:
+        s = svd(x,compute_uv=False)
+        return s[0]/s[-1]
+    else:
+        return norm(x,p)*norm(inv(x),p)
+
 # Generalized inverse
 
 def pinv(a, rcond=1e-15 ):
-    """Return the (Moore-Penrose) pseudo-inverse of a 2-d array
+    """
+    Compute the (Moore-Penrose) pseudo-inverse of a matrix.
+
+    Calculate the generalized inverse of a matrix using its
+    singular-value decomposition (SVD) and including all
+    `large` singular values.
+
+    Parameters
+    ----------
+    a : array_like (M, N)
+      Matrix to be pseudo-inverted.
+    rcond : float
+      Cutoff for `small` singular values.
+      Singular values smaller than rcond*largest_singular_value are
+      considered zero.
+
+    Returns
+    -------
+    B : ndarray (N, M)
+      The pseudo-inverse of `a`. If `a` is an np.matrix instance, then so
+      is `B`.
+
+
+    Raises
+    ------
+    LinAlgError
+      In case SVD computation does not converge.
+
+    Examples
+    --------
+    >>> a = np.random.randn(9, 6)
+    >>> B = np.linalg.pinv(a)
+    >>> np.allclose(a, np.dot(a, np.dot(B, a)))
+    True
+    >>> np.allclose(B, np.dot(B, np.dot(a, B)))
+    True
 
-    This method computes the generalized inverse using the
-    singular-value decomposition and all singular values larger than
-    rcond of the largest.
     """
     a, wrap = _makearray(a)
     _assertNonEmpty(a)
@@ -800,13 +1133,39 @@ def pinv(a, rcond=1e-15 ):
             s[i] = 1./s[i]
         else:
             s[i] = 0.;
-    return wrap(dot(transpose(vt),
-                       multiply(s[:, newaxis],transpose(u))))
+    res = dot(transpose(vt), multiply(s[:, newaxis],transpose(u)))
+    return wrap(res)
 
 # Determinant
 
 def det(a):
-    "The determinant of the 2-d array a"
+    """
+    Compute the determinant of an array.
+
+    Parameters
+    ----------
+    a : array_like, shape (M, M)
+        Input array.
+
+    Returns
+    -------
+    det : ndarray
+        Determinant of `a`.
+
+    Notes
+    -----
+    The determinant is computed via LU factorization using the LAPACK
+    routine z/dgetrf.
+
+    Examples
+    --------
+    The determinant of a 2-D array [[a, b], [c, d]] is ad - bc:
+
+    >>> a = np.array([[1, 2], [3, 4]])
+    >>> np.linalg.det(a)
+    -2.0
+
+    """
     a = asarray(a)
     _assertRank2(a)
     _assertSquareness(a)
@@ -827,26 +1186,92 @@ def det(a):
     sign = add.reduce(pivots != arange(1, n+1)) % 2
     return (1.-2.*sign)*multiply.reduce(diagonal(a), axis=-1)
 
+
 # Linear Least Squares
 
 def lstsq(a, b, rcond=-1):
-    """returns x,resids,rank,s
-    where x minimizes 2-norm(|b - Ax|)
-    resids is the sum square residuals
-    rank is the rank of A
-    s is the rank of the singular values of A in descending order
+    """
+    Return the least-squares solution to an equation.
 
-    If b is a matrix then x is also a matrix with corresponding columns.
-    If the rank of A is less than the number of columns of A or greater than
-    the number of rows, then residuals will be returned as an empty array
-    otherwise resids = sum((b-dot(A,x)**2).
-    Singular values less than s[0]*rcond are treated as zero.
-"""
+    Solves the equation `a x = b` by computing a vector `x` that minimizes
+    the norm `|| b - a x ||`.
+
+    Parameters
+    ----------
+    a : array_like, shape (M, N)
+        Input equation coefficients.
+    b : array_like, shape (M,) or (M, K)
+        Equation target values.  If `b` is two-dimensional, the least
+        squares solution is calculated for each of the `K` target sets.
+    rcond : float, optional
+        Cutoff for ``small`` singular values of `a`.
+        Singular values smaller than `rcond` times the largest singular
+        value are  considered zero.
+
+    Returns
+    -------
+    x : ndarray, shape(N,) or (N, K)
+         Least squares solution.  The shape of `x` depends on the shape of
+         `b`.
+    residues : ndarray, shape(), (1,), or (K,)
+        Sums of residues; squared Euclidian norm for each column in
+        `b - a x`.
+        If the rank of `a` is < N or > M, this is an empty array.
+        If `b` is 1-dimensional, this is a (1,) shape array.
+        Otherwise the shape is (K,).
+    rank : integer
+        Rank of matrix `a`.
+    s : ndarray, shape(min(M,N),)
+        Singular values of `a`.
+
+    Raises
+    ------
+    LinAlgError
+        If computation does not converge.
+
+    Notes
+    -----
+    If `b` is a matrix, then all array results returned as
+    matrices.
+
+    Examples
+    --------
+    Fit a line, ``y = mx + c``, through some noisy data-points:
+
+    >>> x = np.array([0, 1, 2, 3])
+    >>> y = np.array([-1, 0.2, 0.9, 2.1])
+
+    By examining the coefficients, we see that the line should have a
+    gradient of roughly 1 and cuts the y-axis at more-or-less -1.
+
+    We can rewrite the line equation as ``y = Ap``, where ``A = [[x 1]]``
+    and ``p = [[m], [c]]``.  Now use `lstsq` to solve for `p`:
+
+    >>> A = np.vstack([x, np.ones(len(x))]).T
+    >>> A
+    array([[ 0.,  1.],
+           [ 1.,  1.],
+           [ 2.,  1.],
+           [ 3.,  1.]])
+
+    >>> m, c = np.linalg.lstsq(A, y)[0]
+    >>> print m, c
+    1.0 -0.95
+
+    Plot the data along with the fitted line:
+
+    >>> import matplotlib.pyplot as plt
+    >>> plt.plot(x, y, 'o', label='Original data', markersize=10)
+    >>> plt.plot(x, m*x + c, 'r', label='Fitted line')
+    >>> plt.legend()
+    >>> plt.show()
+
+    """
     import math
-    a = asarray(a)
+    a, _ = _makearray(a)
     b, wrap = _makearray(b)
-    one_eq = len(b.shape) == 1
-    if one_eq:
+    is_1d = len(b.shape) == 1
+    if is_1d:
         b = b[:, newaxis]
     _assertRank2(a, b)
     m  = a.shape[0]
@@ -895,7 +1320,7 @@ def lstsq(a, b, rcond=-1):
     if results['info'] > 0:
         raise LinAlgError, 'SVD did not converge in Linear Least Squares'
     resids = array([], t)
-    if one_eq:
+    if is_1d:
         x = array(ravel(bstar)[:n], dtype=result_t, copy=True)
         if results['rank'] == n and m > n:
             resids = array([sum((ravel(bstar)[n:])**2)], dtype=result_t)
@@ -904,39 +1329,46 @@ def lstsq(a, b, rcond=-1):
         if results['rank'] == n and m > n:
             resids = sum((transpose(bstar)[n:,:])**2, axis=0).astype(result_t)
     st = s[:min(n, m)].copy().astype(_realType(result_t))
-    return wrap(x), resids, results['rank'], st
+    return wrap(x), wrap(resids), results['rank'], st
 
 def norm(x, ord=None):
-    """ norm(x, ord=None) -> n
-
+    """
     Matrix or vector norm.
 
-    Inputs:
+    Parameters
+    ----------
+    x : array_like, shape (M,) or (M, N)
+        Input array.
+    ord : {int, 1, -1, 2, -2, inf, -inf, 'fro'}
+        Order of the norm (see table under ``Notes``).
 
-      x -- a rank-1 (vector) or rank-2 (matrix) array
-      ord -- the order of the norm.
+    Returns
+    -------
+    n : float
+        Norm of the matrix or vector
 
-     Comments:
-       For arrays of any rank, if ord is None:
-         calculate the square norm (Euclidean norm for vectors,
-         Frobenius norm for matrices)
+    Notes
+    -----
+    For values ord < 0, the result is, strictly speaking, not a
+    mathematical 'norm', but it may still be useful for numerical
+    purposes.
 
-       For vectors ord can be any real number including Inf or -Inf.
-         ord = Inf, computes the maximum of the magnitudes
-         ord = -Inf, computes minimum of the magnitudes
-         ord is finite, computes sum(abs(x)**ord,axis=0)**(1.0/ord)
+    The following norms can be calculated:
 
-       For matrices ord can only be one of the following values:
-         ord = 2 computes the largest singular value
-         ord = -2 computes the smallest singular value
-         ord = 1 computes the largest column sum of absolute values
-         ord = -1 computes the smallest column sum of absolute values
-         ord = Inf computes the largest row sum of absolute values
-         ord = -Inf computes the smallest row sum of absolute values
-         ord = 'fro' computes the frobenius norm sqrt(sum(diag(X.H * X),axis=0))
+    =====  ============================  ==========================
+    ord    norm for matrices             norm for vectors
+    =====  ============================  ==========================
+    None   Frobenius norm                2-norm
+    'fro'  Frobenius norm                --
+    inf    max(sum(abs(x), axis=1))      max(abs(x))
+    -inf   min(sum(abs(x), axis=1))      min(abs(x))
+    1      max(sum(abs(x), axis=0))      as below
+    -1     min(sum(abs(x), axis=0))      as below
+    2      2-norm (largest sing. value)  as below
+    -2     smallest singular value       as below
+    other  --                            sum(abs(x)**ord)**(1./ord)
+    =====  ============================  ==========================
 
-       For values ord < 0, the result is, strictly speaking, not a
-       mathematical 'norm', but it may still be useful for numerical purposes.
     """
     x = asarray(x)
     nd = len(x.shape)
